@@ -1,11 +1,13 @@
 /* App.m
  * Application class of Cenon
  *
- * Copyright (C) 1995-2005 by vhf interservice GmbH
+ * Copyright (C) 1995-2009 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1995-08-10
- * modified: 2005-01-13
+ * modified: 2009-06-24
+ *           2009-03-27 (-sendEvent: ',' -> '.' on Apple)
+ *           2009-02-25 (-listFromFile: check extensions for any case)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -54,7 +56,7 @@
 @implementation App
 
 /*
- * modified: 2004-08-04
+ * modified: 2006-12-11
  * Initializes the defaults.
  */
 + (void)initialize
@@ -63,16 +65,13 @@
     [registrationDict setObject:@"." forKey:@"NSDecimalSeparator"];
 
     /* General Preferences defaults */
-#ifdef __APPLE__
-    [registrationDict setObject:@"NO"  forKey:@"doCaching"];
-#else
     [registrationDict setObject:@"YES" forKey:@"doCaching"];
-#endif
     [registrationDict setObject:@"0"   forKey:@"unit"];
     [registrationDict setObject:@"NO"  forKey:@"removeBackups"];
     [registrationDict setObject:@"NO"  forKey:@"expertMode"];
-    [registrationDict setObject:@"0"   forKey:@"snap"];
+    [registrationDict setObject:@"2"   forKey:@"snap"];
     [registrationDict setObject:@"0"   forKey:@"lineWidth"];
+    [registrationDict setObject:@"YES"  forKey:@"selectByBorder"];
     [registrationDict setObject:@"20"  forKey:@"cacheLimit"];
 
     /* Import preferences defaults */
@@ -109,36 +108,73 @@
 - (NSString*)currentDirectory
 {   NSString	*cdir = [[self currentDocument] directory], *path;
 
-    path = (cdir && [cdir length]) ? cdir
-                                   : (haveOpenedDocument ? [((NSOpenPanel*)[NSOpenPanel openPanel]) directory]
-                                                         : vhfHomeDirectory());
+    if (cdir && [cdir length])
+        path = cdir;
+    else
+        path = (haveOpenedDocument ? [((NSOpenPanel*)[NSOpenPanel openPanel]) directory]
+                                   : NSHomeDirectory());
     if (!path)
-        return vhfHomeDirectory();
+        return NSHomeDirectory();
     return path;
 }
 
 /* set the current document without regard to the active window
  * we need this in the moment of opening a new document.
  */
-- (void)setCurrentDocument:docu
+- (void)setCurrentDocument:(Document*)docu
 {
-    fixedDocument = docu; 
+    fixedDocument = docu;
+}
+
+- (void)setActiveDocWindow:(DocWindow*)win
+{
+    activeWindowNum = [win windowNumber];
 }
 
 - (Document*)currentDocument
 {
     if (fixedDocument)
-        return(fixedDocument);
-    else if ( [[self mainWindow] isMemberOfClass:[DocWindow class]] )
+        return fixedDocument;
+    /* this is unreliable, because a panel may become the main window! */
+    if ( [[self mainWindow] isMemberOfClass:[DocWindow class]] )
     {	id	docu = [(DocWindow*)[self mainWindow] document];
 
         if ([docu isMemberOfClass:[Document class]])
             return docu;
     }
+    {   NSArray *wins = [self windows];
+        int     i, cnt = [wins count];
+
+        for (i=0; i<cnt; i++)
+            if ( [[wins objectAtIndex:i] windowNumber] == activeWindowNum )
+                return [[wins objectAtIndex:i] document];
+    }
+    /* does this really return the window last worked at? Probably we should remember the active document */
+    /*for ( i=[[self windows] count]-1; i >= 0; i-- )
+    {	DocWindow  *win = [[self windows] objectAtIndex:i];
+
+        if ( [win isMemberOfClass:[DocWindow class]] )
+            return [win document];
+    }*/
+    /*{   int wins[10];
+
+        //NSCountWindowsForContext([self context], &nWin);
+        NSWindowListForContext([self context], 10, wins);   // how to obtain the context? What context is that?
+        for ( i=0; i < 10; i++ )
+        {   DocWindow  *win = wins[i];
+
+            if ( [win isMemberOfClass:[DocWindow class]] )
+                return [win document];
+        }
+    }*/
     return nil;
 }
+- (Document*)openedDocument
+{
+    return document;
+}
 
-- documentInWindow:window
+- (Document*)documentInWindow:(NSWindow*)window
 {
     if ([window isMemberOfClass:[DocWindow class]])
     {	id	docu = [(DocWindow*)window document];
@@ -166,7 +202,7 @@
 }
 
 /* created:  2002-07-18
- * modified: 2005-01-04
+ * modified: 2005-09-05 (load module only once)
  */
 - (void)loadModules
 {   NSBundle		*mainBundle = [NSBundle mainBundle], *bundle;
@@ -175,10 +211,11 @@
     int			i, j;
     NSFileManager	*fileManager = [NSFileManager defaultManager];
     BOOL		camModuleLoaded = NO;
+    NSMutableArray	*loadedFiles = [NSMutableArray array];
 
     /* load modules from application bundle
-     * FIXME: This should be integrated below (case 0)
-     *        Each module should give info about other modules needed first
+     * FIXME: The CAM module should be integrated below (case 0)
+     *        Each module should give info about other modules needed before
      *        Each module should give info about modules to be in conflict
      *        Available modules should be selectable by switches in preferences
      */
@@ -197,7 +234,8 @@
 
         switch (i)
         {
-            //case 0: path = [[mainBundle resourcePath] stringByAppendingPathComponent:BUNDLEFOLDER];
+            //case 0: path = [[mainBundle resourcePath] stringByAppendingPathComponent:BUNDLEFOLDER]; break;
+            //case 0: path = [mainBundle resourcePath]; break;
             case 0: path = [userLibrary()  stringByAppendingPathComponent:@"Bundles"]; break;
             case 1: path = [localLibrary() stringByAppendingPathComponent:@"Bundles"]; break;
             case 2: path = userBundlePath(); break;
@@ -210,14 +248,16 @@
         for (j=0; j<(int)[files count]; j++)
         {   NSString	*file = [files objectAtIndex:j];
 
-            if ([file hasPrefix:@"Cut."] && camModuleLoaded)	// either CAM or Cut, not both
+            if (camModuleLoaded && [file hasPrefix:@"Cut."])	// either CAM or Cut, not both
                 continue;
             if ( [file hasSuffix:@".bundle"] &&
+                 ![loadedFiles containsObject:file] &&	// already loaded ?
                  (bundle = [NSBundle bundleWithPath:[path stringByAppendingPathComponent:file]]) )
             {
                 bundleClass = [bundle principalClass];	// controller
                 [modules addObject:bundle];		// our loaded modules
                 [bundleClass instance];			// create instance
+                [loadedFiles addObject:file];
             }
         }
     }
@@ -228,7 +268,7 @@
 }
 
 /* created:  1993-01-??
- * modified: 2004-02-25
+ * modified: 2005-11-18
  */
 - (void)applicationDidFinishLaunching:(NSNotification *)notification
 {   NSApplication	*theApplication = [notification object];
@@ -251,15 +291,15 @@
             /* create HOME/Cenon */
             [fileManager createDirectoryAtPath:path attributes:nil];
             /* create HOME/Cenon/Projects */
-            [fileManager createDirectoryAtPath:[NSString stringWithFormat:@"%@/Projects", path]
+            [fileManager createDirectoryAtPath:vhfPathWithPathComponents(path, @"Projects", nil)
                                     attributes:nil];
             /* copy Cenon/Projects/.dir.tiff */
-            from = [NSString stringWithFormat:@"%@/Projects/.dir.tiff", localLibrary()];
-            to = [NSString stringWithFormat:@"%@/Projects/.dir.tiff", path];
+            from = vhfPathWithPathComponents(localLibrary(), @"Projects", @".dir.tiff", nil);
+            to = vhfPathWithPathComponents(path, @"Projects", @".dir.tiff", nil);
             [fileManager copyPath:from toPath:to handler:nil];
             /* copy Cenon/dir.tiff */
-            from = [NSString stringWithFormat:@"%@/.dir.tiff", localLibrary()];
-            to = [NSString stringWithFormat:@"%@/.dir.tiff", path];
+            from = vhfPathWithPathComponents(localLibrary(), @".dir.tiff", nil);
+            to = vhfPathWithPathComponents(path, @".dir.tiff", nil);
             [fileManager copyPath:from toPath:to handler:nil];
         }
     }
@@ -272,12 +312,12 @@
     /* load modules */
     [self loadModules];
 
-    /* AppCAM */
-    if ([self respondsToSelector:@selector(xyzInitParameter)])
-        [self performSelector:@selector(xyzInitParameter)];
-
     [self displayToolPanel:YES];
 
+#if !defined(GNUSTEP_BASE_VERSION) && !defined(__APPLE__)	// OpenStep 4.2
+    [self saveAsPanel];	// OS Bug: If 1st Panel is the open panel this function would return an open panel
+#endif
+
     [NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:500.0/1000000.0]];
 }
 
@@ -288,20 +328,24 @@
 }
 
 
-- (void)new:sender
-/*
- * Creates a new document--called by pressing New in the Document menu.
+/* Creates a new document--called by pressing New in the Document menu.
  */
+- (void)new:sender
 {
     document = [Document new];
 }
 
+/* modified: 2009-02-25 (fileNameLC)
+ */
 - (id)listFromFile:(NSString*)fileName
-{   NSArray		*list = nil;
-    NSString		*path;
-    NSFileManager	*defaultManager = [NSFileManager defaultManager];
+{   NSArray         *list = nil;
+    NSString        *path, *name;
+    NSFileManager   *defaultManager = [NSFileManager defaultManager];
+    NSString        *fileNameLC = [fileName lowercaseString];
 
-    if ( [fileName hasSuffix:@".dxf"] )
+    //if ( [fileName rangeOfString:@".dxf" options:NSCaseInsensitiveSearch].length )
+    //if ( [fileName compare:@".dxf" options:NSAnchoredSearch|NSBackwardsSearch|NSCaseInsensitiveSearch] )
+    if ( [fileNameLC hasSuffix:@".dxf"] )
     {	id	dxfImport = [[DXFImportSub allocWithZone:[self zone]] init];
         NSData	*data = [NSData dataWithContentsOfFile:fileName];	// get data object
 
@@ -309,16 +353,18 @@
         list = [[dxfImport importDXF:data] retain];	// get list of graphic-objects from import
         [dxfImport release];
     }
-    else if ( [fileName hasSuffix:@".hpgl"] || [fileName hasSuffix:@".hgl"] || [fileName hasSuffix:@".plt"] )
-    {	id	hpglImport;
+    else if ( [fileNameLC hasSuffix:@".hpgl"] || [fileNameLC hasSuffix:@".hgl"] ||
+              [fileNameLC hasSuffix:@".plt"] )
+    {	id      hpglImport;
         NSData	*data;
 
         /* load parameter file
          * 1st try it in the users home library then try it in /LocalLibrary
          */
-        path = [NSString stringWithFormat:@"%@/%@/%@.dev", userLibrary(), HPGLPATH, Prefs_HPGLParmsFileName];
+        name = [Prefs_HPGLParmsFileName stringByAppendingPathExtension:DEV_EXT];
+        path = vhfPathWithPathComponents(userLibrary(), HPGLPATH, name, nil);
         if ( ![defaultManager fileExistsAtPath:path] )
-        {   path = [NSString stringWithFormat:@"%@/%@/%@.dev", localLibrary(), HPGLPATH, Prefs_HPGLParmsFileName];
+        {   path = vhfPathWithPathComponents(localLibrary(), HPGLPATH, name, nil);
             if ( ![defaultManager fileExistsAtPath:path] )
             {	NSRunAlertPanel(@"", CANTLOADFILE_STRING, OK_STRING, nil, nil, path);
                 return nil;
@@ -334,16 +380,17 @@
         list = [[hpglImport importHPGL:data] retain];		// get list of graphic-objects from import
         [hpglImport release];
     }
-    else if ( [fileName hasSuffix:@".ger"] || [fileName hasSuffix:@".gerber"] )
+    else if ( [fileNameLC hasSuffix:@".ger"] || [fileNameLC hasSuffix:@".gerber"] )
     {   id	gerberImport;
         NSData	*data;
 
         /* load parameter file
          * 1st try it in the users home library then try it in /LocalLibrary
          */
-        path = [NSString stringWithFormat:@"%@/%@/%@.dev", userLibrary(), GERBERPATH, Prefs_GerberParmsFileName];
+        name = [Prefs_GerberParmsFileName stringByAppendingPathExtension:DEV_EXT];
+        path = vhfPathWithPathComponents(userLibrary(), GERBERPATH, name, nil);
         if ( ![defaultManager fileExistsAtPath:path] )
-        {   path = [NSString stringWithFormat:@"%@/%@/%@.dev", localLibrary(), GERBERPATH, Prefs_GerberParmsFileName];
+        {   path = vhfPathWithPathComponents(localLibrary(), GERBERPATH, name, nil);
             if ( ![defaultManager fileExistsAtPath:path] )
             {	NSRunAlertPanel(@"", CANTLOADFILE_STRING, OK_STRING, nil, nil, path);
                 return nil;
@@ -373,11 +420,10 @@
                 {   [gerberImport release];
                     return nil;
                 }
-                path = [NSString stringWithFormat:@"%@/%@/%@.tab", userLibrary(), GERBERPATH, Prefs_GerberParmsFileName];
+                name = [Prefs_GerberParmsFileName stringByAppendingPathExtension:@"tab"];
+                path = vhfPathWithPathComponents(userLibrary(), GERBERPATH, name, nil);
                 if ( ![gerberImport loadApertures:path] )
-                {
-                    path = [NSString stringWithFormat:@"%@/%@/%@.tab",
-                           localLibrary(), GERBERPATH, Prefs_GerberParmsFileName];
+                {   path = vhfPathWithPathComponents(localLibrary(), GERBERPATH, name, nil);
                     if ( ![gerberImport loadApertures:path] )
                     {   NSRunAlertPanel(@"", CANTLOADFILE_STRING, OK_STRING, nil, nil, path);
                         [gerberImport release];
@@ -389,7 +435,7 @@
         list = [[gerberImport importGerber:data] retain];	// get list of graphic-objects from import
         [gerberImport release];
     }
-    else if ( [fileName hasSuffix:@".drl"] || [fileName hasSuffix:@".din"] )
+    else if ( [fileNameLC hasSuffix:@".drl"] || [fileNameLC hasSuffix:@".din"] )
     {   id		dinImport;
         NSData		*data;
         NSString	*devFileName;
@@ -399,9 +445,10 @@
          */
         if ([(devFileName = Prefs_DINParmsFileName) length])
         {
-            path = [NSString stringWithFormat:@"%@/%@/%@.dev", userLibrary(), DINPATH, devFileName];
+            name = [devFileName stringByAppendingPathExtension:DEV_EXT];
+            path = vhfPathWithPathComponents(userLibrary(), DINPATH, name, nil);
             if ( ![defaultManager fileExistsAtPath:path] )
-            {   path = [NSString stringWithFormat:@"%@/%@/%@.dev", localLibrary(), DINPATH, devFileName];
+            {   path = vhfPathWithPathComponents(localLibrary(), DINPATH, name, nil);
                 if ( ![defaultManager fileExistsAtPath:path] )
                 {   NSRunAlertPanel(@"", CANTLOADFILE_STRING, OK_STRING, nil, nil, path);
                     return nil;
@@ -422,16 +469,16 @@
         list = [[DINImportSub layerListFromGraphicList:list] retain];
         [dinImport release];
     }
-    else if ( [fileName hasSuffix:@".font"] || [fileName hasSuffix:@".pfa"] || [fileName hasSuffix:@".pfb"] )
-    {   id		fontObject, type1Import;
+    else if ( [fileNameLC hasSuffix:@".font"] || [fileNameLC hasSuffix:@".pfa"] || [fileNameLC hasSuffix:@".pfb"] )
+    {   id          fontObject, type1Import;
         NSData		*data;
         NSString	*name;
 
-	if ( [fileName hasSuffix:@".pfa"] || [fileName hasSuffix:@".pfb"] )	// pfa file
+        if ( [fileNameLC hasSuffix:@".pfa"] || [fileNameLC hasSuffix:@".pfb"] )	// pfa file
             name = [NSString stringWithFormat:@"%@", fileName];
-	else
+        else
             name = [fileName stringByAppendingPathComponent:
-                [[fileName lastPathComponent] stringByDeletingPathExtension]];
+        [[fileName lastPathComponent] stringByDeletingPathExtension]];
         type1Import = [[Type1ImportSub allocWithZone:[self zone]] init];	// get new import-object
         data = [NSData dataWithContentsOfFile:name];				// get file-data
 
@@ -441,12 +488,12 @@
     }
 
     /* PostScript, AI */
-    else if ( [fileName hasSuffix:@".eps"] || [fileName hasSuffix:@".ps"] ||
-              [fileName hasSuffix:@".ai"] )
+    else if ( [fileNameLC hasSuffix:@".eps"] || [fileNameLC hasSuffix:@".ps"] ||
+              [fileNameLC hasSuffix:@".ai"] )
         list = [[self listFromPSFile:fileName] retain];
 
     /* PDF */
-    else if ( [fileName hasSuffix:@".pdf"] || [fileName hasSuffix:@".PDF"])
+    else if ( [fileNameLC hasSuffix:@".pdf"] || [fileNameLC hasSuffix:@".PDF"])
     {   PSImportSub	*psImport = [[PSImportSub allocWithZone:[self zone]] init];
 
         [psImport preferArcs:Prefs_PSPreferArcs];
@@ -504,10 +551,20 @@
     return [psImport importPS:data];
 }
 
-- (int)openFile:(NSString *)fileName
+- (BOOL)openFile:(NSString *)fileName
 {   id			list = 0;
     NSString		*path;
+    NSFileManager	*fileManager = [NSFileManager defaultManager];
 
+    if (!appIsRunning)	// give application a chance to load modules first
+    {
+        [self performSelector:@selector(openFile:) withObject:fileName afterDelay:0];
+        return YES;
+    }
+
+    document = nil;
+    if ( ![fileManager fileExistsAtPath:fileName] )
+        return NO;
     if ( [fileName hasSuffix:DOCUMENT_EXT] )
     {	int	i;
 
@@ -516,14 +573,15 @@
 
             if (!docu)
                 continue;
-            path = [NSString stringWithFormat:@"%@/%@", [docu directory], [docu name]];
+            path = vhfPathWithPathComponents([docu directory], [docu name], nil);
             if ( docu && [path isEqualToString:fileName] )
             {	[[[self windows] objectAtIndex:i] makeKeyAndOrderFront:self];
                 return YES;
             }
         }
         document = [Document newFromFile:fileName];
-        [self setCurrentDocument:0];
+        [self setCurrentDocument:nil];
+        return YES;
     }
     else if ( (list = [self listFromFile:fileName]) )
     {
@@ -531,69 +589,83 @@
         if ( [fileName hasSuffix:@".font"] || [fileName hasSuffix:@".pfa"] || [fileName hasSuffix:@".pfb"] )
         {
             document = [Document new];
-            [self setCurrentDocument:0];
+            [self setCurrentDocument:nil];
             [document setName:UNTITLED_STRING andDirectory:[self currentDirectory]];
             [document setDirty:YES];
-            [document setFontObject:list]; // font daten + font list hinzufgen
+            [document setFontObject:list]; // font daten + font list hinzufˆgen
         }
         /* imports with complete layerList or a single list of objects */
         else
         {
             document = [Document newFromList:list];
-            [self setCurrentDocument:0];
+            [self setCurrentDocument:nil];
         }
+        return YES;
     }
-    return YES;
+    return NO;
 }
 
-/*
+/* Import files
+ * modified: 20005-09-25
  */
+typedef enum
+{   IMPORTTO_SELECTEDLAYER  = 0,
+    IMPORTTO_NEWLAYER       = 1,
+    IMPORTTO_EXISTINGLAYERS = 2
+} ImportToLayerSelection;
 - (void)import:sender
-{   NSArray		*fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
+{   NSArray     *fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
                                               @"dxf", @"eps", @"ps", @"pdf", @"gerber", @"ger",
                                               @"cenon", @"tiff", @"jpg", @"gif",
                                               @"font", @"pfa", @"din", @"drl", nil];
-    NSString		*fileName;
-    id			openPanel = [NSOpenPanel openPanel];
-    static NSString	*openDir = @"";
+    NSString    *fileName;
+    id          openPanel = [NSOpenPanel openPanel];
+    static      NSString	*openDir = @"";
 
     [openPanel setAccessoryView:[importAccessory retain]];
     [openPanel setAllowsMultipleSelection:NO];
 
     if ( [openPanel runModalForDirectory:openDir file:@"" types:fileTypes] )
-    {   int	createNewLayer = [iaPopup indexOfSelectedItem];
-        id	list;
-        DocView	*view = [[self currentDocument] documentView];
+    {   ImportToLayerSelection	importTo = [iaPopup indexOfSelectedItem];
+        id      list;
+        DocView *view = [[self currentDocument] documentView];
 
         [openDir release];
         openDir = [[[openPanel filename] stringByDeletingLastPathComponent] retain];
 
-	fileName = [openPanel filename];
+        fileName = [openPanel filename];
         list = [self listFromFile:fileName];
 
-        if (createNewLayer)
-            [view addList:list toLayerAtIndex:-1];
-        else
-        {   id	layerList = [view layerList];
-            int	i = [view indexOfSelectedLayer];
+        switch (importTo)
+        {
+            case IMPORTTO_NEWLAYER:
+                [view addList:list toLayerAtIndex:-1 /*replaceObjects:NO*/];
+                break;
+            case IMPORTTO_SELECTEDLAYER:
+            {   id	layerList = [view layerList];
+                int	i = [view indexOfSelectedLayer];
 
-            if ([[layerList objectAtIndex:i] editable])
-                [view addList:list toLayerAtIndex:i];
-            else
-                NSRunAlertPanel(@"", LAYERNOTEDITABLE_STRING, OK_STRING, nil, nil, NULL);
-            /*for (i=0; i<[layerList count]; i++)
                 if ([[layerList objectAtIndex:i] editable])
+                    [view addList:list toLayerAtIndex:i /*replaceObjects:NO*/];
+                else
+                    NSRunAlertPanel(@"", LAYERNOTEDITABLE_STRING, OK_STRING, nil, nil, NULL);
+                /*for (i=0; i<[layerList count]; i++)
+                    if ([[layerList objectAtIndex:i] editable])
                 {   [view addList:list toLayerAtIndex:i];
-                    break;
+                        break;
                 }
-            if (i>=[layerList count])
-                [view addList:list toLayerAtIndex:-1];*/
-        }
+                if (i>=[layerList count])
+                    [view addList:list toLayerAtIndex:-1];*/
+                break;
+            }
+            case IMPORTTO_EXISTINGLAYERS:
+                [view addList:list toLayerAtIndex:-2 /*replaceObjects:YES*/];
+         }
     }
     [openPanel setAccessoryView:nil];
 }
 
-/*
+/* modified: 2005-11-14
  */
 - (void)importASCII:sender
 {   NSArray		*fileTypes = [NSArray arrayWithObjects:@"txt", @"asc", @"tab", nil];
@@ -607,15 +679,19 @@
     [openPanel setAllowsMultipleSelection:NO];
     if ( [openPanel runModalForDirectory:openDir file:@"" types:fileTypes] )
     {   int		sort = [iaaRadio selectedColumn];
-        NSString	*tabName = ([iaaPopup indexOfSelectedItem]<1) ? nil : [NSString stringWithFormat:@"%@%@", [iaaPopup title], DICT_EXT], *string;
-        NSDictionary	*conversionDict = dictionaryFromFolder(CHARCONV_FOLDER, tabName);
+        NSString	*tabName, *string;
+        NSDictionary	*conversionDict = nil;
 
+        if ( [iaaPopup indexOfSelectedItem] >= 1 &&
+             (tabName = [NSString stringWithFormat:@"%@%@", [iaaPopup title], DICT_EXT]) )
+                conversionDict = dictionaryFromFolder(CHARCONV_FOLDER, tabName);
         [openDir release];
         openDir = [[[openPanel filename] stringByDeletingLastPathComponent] retain];
 
 	fileName = [openPanel filename];
         string = [NSString stringWithContentsOfFile:fileName];
-        [[[self currentDocument] documentView] importASCII:stringWithConvertedChars(string, conversionDict) sort:sort];
+        [[[self currentDocument] documentView] importASCII:stringWithConvertedChars(string, conversionDict)
+                                                      sort:sort];
     }
     [openPanel setAccessoryView:nil];
 }
@@ -625,26 +701,25 @@
  * and loads the specified file into it.
  */
 - (void)openDocument:sender
-{   NSArray		*fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
+{   NSArray         *fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
                                               @"dxf", @"eps", @"ps", @"pdf", @"gerber", @"ger",
                                               @"cenon", @"tiff", @"jpg", @"gif",
                                               @"font", @"pfa", @"din", @"drl", nil];
-    NSArray		*fileNames;
-    id			openPanel = [NSOpenPanel openPanel];
-    int			i, cnt;
-    static NSString	*openDir = @"";
+    NSArray         *fileNames;
+    id              openPanel = [NSOpenPanel openPanel];
+    int             i, cnt;
+    static NSString *openDir = @"";
 
     [openPanel setAllowsMultipleSelection:YES];
     if ( [openPanel runModalForDirectory:openDir file:@"" types:fileTypes] )
-    //if ([openPanel runModalForTypes:fileTypes])
     {
         [openDir release];
         openDir = [[[openPanel filename] stringByDeletingLastPathComponent] retain];
 
-	fileNames = [openPanel filenames];
+        fileNames = [openPanel filenames];
         cnt = [fileNames count];
-	for ( i=0; i<cnt; i++ )
-	    haveOpenedDocument = [self openFile:[fileNames objectAtIndex:i]] || haveOpenedDocument;
+        for ( i=0; i<cnt; i++ )
+            haveOpenedDocument = [self openFile:[fileNames objectAtIndex:i]] || haveOpenedDocument;
     }
 }
 
@@ -723,7 +798,7 @@
 {   NSSavePanel	*savePanel = [NSSavePanel savePanel];
 
     /* we have to set the file type and accessory on every call,
-     * because it gets destroyed on a call of saveOutput !
+     * because it gets destroyed sometimes !
      */
     [savePanel setAccessoryView:[savePanelAccessory retain]];
     //[self changeSaveType:spaFormatPopUp];
@@ -765,8 +840,6 @@
         [infoPanel orderFront:self];
     }
 #endif
-    //if ( !dataPanel )
-    //    [self showDataPanel:self];
 
     if (![self openFile:path])
         NSRunAlertPanel(@"", CANTOPENFILE_STRING, OK_STRING, nil, nil);
@@ -786,10 +859,22 @@
     {
         if (![NSBundle loadModelNamed:@"Info" owner:self])
             NSLog(@"Cannot load Info interface file");
+#ifdef GNUSTEP_BASE_VERSION // FIXME: NSApplication on GNUstep has an icar named "_infoPanel" (2009-06-24)
+        if ( !infoPanel && [self valueForKey:@"_infoPanel"] )
+            infoPanel = [self valueForKey:@"_infoPanel"];
+#endif
     }
 
     [serialNumber setStringValue:@""];
 
+    /*{   NSDictionary    *infoDict = [[NSBundle mainBundle] infoDictionary];
+        NSString    *version;
+
+        //NSLog(@"%@", infoDict);
+        if ( (version=[infoDict objectForKey:@"CFBundleVersion"]) )
+            [infoVersionNo setStringValue:version]; // ex: 3.9.0 pre 3 (2008-03-01)
+    }*/
+
     [[NSNotificationCenter defaultCenter] postNotificationName:InfoPanelWillDisplay
                                                         object:nil userInfo:nil];
 
@@ -802,8 +887,9 @@
     [self displayInfo];
     [infoPanel makeKeyAndOrderFront:sender];
 }
-- (id)infoVersionText	{ return kindOfVersion; }
-- (id)infoSerialText	{ return serialNumber; }
+- (id)infoVersionNo     { return infoVersionNo; }   // ex: 3.9.0
+- (id)infoVersionText	{ return kindOfVersion; }   // ex: "Licensed version"
+- (id)infoSerialText	{ return serialNumber; }    // ex: 020001
 
 - (void)showPrefsPanel:sender
 {
@@ -822,16 +908,49 @@
     return preferencesPanel;
 }
 
-#if 0
+/* load PDF documentation
+ * Note: each module can add a help menu entry to load it's docu
+ * modified: 2007-07-22
+ */
+#if defined(GNUSTEP_BASE_VERSION) || defined(__APPLE__)	// GNUstep or Apple
 - (void)showHelp:sender
-{
-    if (!helpPanel)
+{   NSArray     *localizations;
+    NSString    *locale, *path, *helpFile;
+    int         l, i;
+
+    localizations = [NSBundle preferredLocalizationsFromArray:[[NSBundle mainBundle] localizations]];
+
+    for (l=0; l<[localizations count]; l++ )
     {
-        if (![NSBundle loadModelNamed:@"Help" owner:self])
-            NSLog(@"Cannot load Help interface file");	
+        locale = [[localizations objectAtIndex:l] stringByAppendingString:@".lproj"];
+        for (i=0; i<3; i++)
+        {
+            switch (i)
+            {
+                case 0: path = [[NSBundle mainBundle] resourcePath];
+                    helpFile = vhfPathWithPathComponents(path, locale, @"Cenon.pdf", nil);
+                    break;
+                case 1: path = vhfUserLibrary(APPNAME);
+                    helpFile = vhfPathWithPathComponents(path, @"Docu", locale, @"Cenon.pdf", nil);
+                    break;
+                case 2: path = vhfLocalLibrary(APPNAME);
+                    helpFile = vhfPathWithPathComponents(path, @"Docu", locale, @"Cenon.pdf", nil);
+                    break;
+                default:
+                    return;
+            }
+            if ( [[NSFileManager defaultManager] fileExistsAtPath:helpFile] )
+            {   [[NSWorkspace sharedWorkspace] openFile:helpFile];
+                l = [localizations count];
+                break;
+            }
+        }
     }
+
+    /*if ( !helpPanel && ![[NSBundle mainBundle] loadModelNamed:@"Help" owner:self] )
+            NSLog(@"Cannot load Help interface file");
     [helpPanel setFrameAutosaveName:@"HelpPanel"];
-    [helpPanel makeKeyAndOrderFront:sender];
+    [helpPanel makeKeyAndOrderFront:sender];*/
 }
 #endif
 
@@ -848,7 +967,7 @@
     [inspectorPanel setBecomesKeyOnlyIfNeeded:YES];
     [inspectorPanel orderFront:sender];
 }
-- inspectorPanel;
+- inspectorPanel
 {
     return inspectorPanel;
 }
@@ -864,7 +983,7 @@
     [transformPanel setFrameAutosaveName:@"TransformPanel"];
     [transformPanel makeKeyAndOrderFront:sender];
 }
-- transformPanel;
+- transformPanel
 {
     return transformPanel;
 }
@@ -875,12 +994,11 @@
     {
         if (![NSBundle loadModelNamed:@"ProjectSettingsPanel" owner:self])
             NSLog(@"Cannot load ProjectSettingsPanel interface file");	
-        [projectSettingsPanel init];
     }
     //[projectSettingsPanel setFrameAutosaveName:@"ProjectSettingsPanel"];
     [projectSettingsPanel makeKeyAndOrderFront:sender];
 }
-- projectSettingsPanel;
+- projectSettingsPanel
 {
     return projectSettingsPanel;
 }
@@ -1094,13 +1212,34 @@
 - (BOOL)control		{ return control; }
 - (BOOL)alternate	{ return alternate; }
 
-/*
- * modified: 05.11.95
+/* created:  1995-11-05
+ * modified: 2009-03-27
  *
  * We override this because we need to find out when the command key is down
+ * and to change ',' to '.' for the decimal separator on the numeric pad
  */
 - (void)sendEvent:(NSEvent *)event
 {
+#ifdef __APPLE__
+    /* Change ',' to '.' */
+    if ( event && [event type] == NSKeyDown && [event keyCode] == 65 )    // decimal-key: we want a '.'
+    {   NSString    *chars = [event charactersIgnoringModifiers];
+
+        chars = @".";   // we change ',' to '.'
+        event = [NSEvent keyEventWithType:[event type]
+                                 location:[event locationInWindow]
+                            modifierFlags:[event modifierFlags]
+                                timestamp:[event timestamp]
+                             windowNumber:[event windowNumber]
+                                  context:[event context]
+                               characters:chars
+              charactersIgnoringModifiers:chars
+                                isARepeat:[event isARepeat]
+                                  keyCode:[event keyCode]];
+    }
+#endif
+
+    /* find out if command is pressed */
     if (event && [event type] < NSAppKitDefined)
     {	BOOL	lastCommand = command;
 
@@ -1112,13 +1251,13 @@
         /* temporary set arrow mode
          */
         if (command != lastCommand)
-            [self setCurrent2DTool:(command) ? nil : self];
+            [self setCurrent2DTool:(command) ? nil : (id)self];
 
         if (command && [event type] == NSKeyDown &&
             [[self mainWindow] isMemberOfClass:[DocWindow class]])
-        {   NSString	*string = [event charactersIgnoringModifiers];
-            int		i = [string intValue];
-            id		fe = [[self mainWindow] fieldEditor:NO forObject:nil];
+        {   NSString    *string = [event charactersIgnoringModifiers];
+            int         i = [string intValue];
+            id          fe = [[self mainWindow] fieldEditor:NO forObject:nil];
 
             /* set inspector panel only, if no text ruler is activated,
              * to avoid setting the inspector when rulers are copied
@@ -1149,8 +1288,7 @@
           action == @selector(importASCII:) ||
           action == @selector(revertToSaved:) ||
           action == @selector(save:) ||
-          action == @selector(saveAs:) ||
-          action == @selector(saveOutput:)) &&
+          action == @selector(saveAs:)) &&
          ![self currentDocument] )
         return NO;
 
