@@ -1,11 +1,11 @@
 /* VCurve.m
  * 2-D Bezier curve
  *
- * Copyright (C) 1996-2006 by vhf interservice GmbH
+ * Copyright (C) 1996-2008 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-29
- * modified: 2006-11-24
+ * modified: 2008-08-06 (-getListOfObjectsSplittedFrom::)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -524,16 +524,16 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
     float	angle;
 
     p = NSMakePoint(0.0, 0.0);
-    gradA = [self gradientAt:0.0];			/* gradient of start-point object */
-    gradB = [self gradientAt:1.0/3.0];			/* gradient of end-point for object */
+    gradA = [self gradientAt:0.0];          // gradient of start-point object
+    gradB = [self gradientAt:1.0/3.0];      // gradient of end-point for object
     t1End.x = p.x - gradA.x;
     t1End.y = p.y - gradA.y;
     t2End.x = p.x + gradB.x;
     t2End.y = p.y + gradB.y;
-    angle = vhfAngleBetweenPoints(t1End, p, t2End);	/* get angle (ccw) on right side */
+    angle = vhfAngleBetweenPoints(t1End, p, t2End); // get angle (ccw) on right side
 
-    gradA = [self gradientAt:2.0/3.0];			/* gradient of start-point object */
-    gradB = [self gradientAt:1.0];			/* gradient of end-point for object */
+    gradA = [self gradientAt:2.0/3.0];      // gradient of start-point object
+    gradB = [self gradientAt:1.0];          // gradient of end-point for object
     t1End.x = p.x - gradA.x;
     t1End.y = p.y - gradA.y;
     t2End.x = p.x + gradB.x;
@@ -549,10 +549,15 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
 #endif
 
     {   NSPoint	pArray[2];
+        float	a1, a2;
 
-        if ( p3.x==p2.x && p3.y==p2.y )
+        if ( (Diff(p3.x, p2.x)<=TOLERANCE && Diff(p3.y, p2.y)<=TOLERANCE) ||
+             (Diff(p0.x, p2.x)<=TOLERANCE && Diff(p0.y, p2.y)<=TOLERANCE) )
             return ( vhfAngleBetweenPoints(p3, p1, p0) + 180.0 );
-        if ( (p2.x==p1.x && p2.y==p1.y) || (p1.x==p0.x && p1.y==p0.y) )
+        if ( (Diff(p2.x, p1.x)<=TOLERANCE && Diff(p2.y, p1.y)<=TOLERANCE) ||
+             (Diff(p1.x, p0.x)<=TOLERANCE && Diff(p1.y, p0.y)<=TOLERANCE) ||
+             (Diff(p3.x, p1.x)<=TOLERANCE && Diff(p3.y, p1.y)<=TOLERANCE) )
+        //if ( (p2.x==p1.x && p2.y==p1.y) || (p1.x==p0.x && p1.y==p0.y) )
             return ( vhfAngleBetweenPoints(p3, p2, p0) + 180.0 );
 
         /* line from p0 to p1 intersect line from p2 to p3 */
@@ -562,7 +567,15 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
             mP.y = Min(p1.y, p2.y) + ((Max(p1.y, p2.y)-Min(p1.y, p2.y))/2.0);
             return vhfAngleBetweenPoints(p3, mP, p0) + 180.0;
         }
-        return vhfAngleBetweenPoints(p3, p2, p1) + vhfAngleBetweenPoints(p2, p1, p0);
+        /* must not be 0 ! */
+        a1 = vhfAngleBetweenPoints(p3, p2, p1);
+        a2 = vhfAngleBetweenPoints(p2, p1, p0);
+        if ( a1 < 1.0 || a2 < 1.0 || a1 > 359.0 || a2 > 359.0 )
+        {
+            if ( a1 < 1.0 || a1 > 359.0 ) a1 = 180.0;
+            if ( a2 < 1.0 || a2 > 359.0 ) a2 = 180.0;
+        }
+        return a1 + a2;
     }
 }
 
@@ -585,8 +598,8 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
 
     /* here we tile the curve
      * 1st we check if curve needs to be splitted at start and end
-     * if the curve isnt splitted on both sides we check the start point
-     * if the curve isnt splitted on start side we check the end point
+     * if the curve isn't split on both sides we check the start point
+     * if the curve isn't split on start side we check the end point
      */
     if ( !(SqrDistPoints(beg, begO) < TOLERANCE) && !(SqrDistPoints(end, endO) < TOLERANCE) )
     {
@@ -862,7 +875,7 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
 
                 [drawCurveGraphic setVertices:p0 :p1 :drawPoint :p3];
                 drawBounds = [drawCurveGraphic extendedBoundsWithScale:[view scaleFactor]];
-                /* the united rect of the two rects we need to redraw the view */
+                /* the united rect of the two rect¬s we need to redraw the view */
                 gridBounds  = NSUnionRect(drawBounds , gridBounds);
 
                 /* if line is not inside view we set it invalid */
@@ -1430,7 +1443,7 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
  *            0, 1, 2, 3
  */
 - (int)intersectLine:(NSPoint*)pArray :(NSPoint)pl0 :(NSPoint)pl1
-{   double	dx, m=0.0, x0=0.0, ax, bx, cx, ay, by, cy, sol[3];
+{   double	dx, dy, m=0.0, x0=0.0, ax, bx, cx, ay, by, cy, sol[3];
     float	max, min;
     int		numSol=0, n, n1;
     NSRect	cBounds, lBounds;
@@ -1468,7 +1481,19 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
      * g(x) = m*x + x(0)
      */
     dx = pl1.x - pl0.x;
-    if ( Diff(dx, 0) > TOLERANCE )
+    dy = pl1.y - pl0.y;
+    /* line is vertical
+     * x(t) = ax*t^3 + bx*t^2 + cx*t + x(0)
+     * with x(t) = x of line
+     */
+    if ( Diff(dx, 0) < TOLERANCE || (Diff(dx, 0) < Diff(dy, 0) && Diff(dx, 0) < 5.0*TOLERANCE) )
+    {	n1 = svPolynomial3( ax, bx, cx, p0.x-pl0.x, sol);
+
+        /* used to test, if points are between the endpoints of our line */
+        min = Min(pl0.y, pl1.y) - 2.0*TOLERANCE;
+        max = Max(pl0.y, pl1.y) + 2.0*TOLERANCE;
+    }
+    else // if ( Diff(dx, 0) > TOLERANCE )
     {	m  = (pl1.y - pl0.y) / dx;
         x0 = pl0.y - m * pl0.x;
 
@@ -1484,17 +1509,6 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
         min = Min(pl0.x, pl1.x) - 2.0*TOLERANCE;
         max = Max(pl0.x, pl1.x) + 2.0*TOLERANCE;
     }
-    /* line is vertical
-     * x(t) = ax*t^3 + bx*t^2 + cx*t + x(0)
-     * with x(t) = x of line
-     */
-    else
-    {	n1 = svPolynomial3( ax, bx, cx, p0.x-pl0.x, sol);
-
-        /* used to test, if points are between the endpoints of our line */
-        min = Min(pl0.y, pl1.y) - 2.0*TOLERANCE;
-        max = Max(pl0.y, pl1.y) + 2.0*TOLERANCE;
-    }
 
     for (n = 0; n < n1; n++)
     {	NSPoint	p, gc, gl;
@@ -1509,20 +1523,21 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
             if ( Diff(p.x, p0.x) + Diff(p.y, p0.y) > TOLERANCE)
                 continue;
         }
-        if ( Diff(dx, 0) > TOLERANCE )
-        {   points->x = ax*sol[n]*sol[n]*sol[n] + bx*sol[n]*sol[n] + cx*sol[n] + p0.x;
-            if ( points->x >= min && points->x <= max)
-            {	points->y = m*points->x+x0;
+        /* vertical line */
+        if ( Diff(dx, 0) < TOLERANCE || (Diff(dx, 0) < Diff(dy, 0) && Diff(dx, 0) < 5.0*TOLERANCE) )
+        {   points->y = ay*sol[n]*sol[n]*sol[n] + by*sol[n]*sol[n] + cy*sol[n] + p0.y;
+            if ( points->y >= min && points->y <= max)
+            {	points->x = pl0.x;
                 points++;
                 numSol++;
             }
             else
                 continue;
         }
-        else	/* vertical line */
-        {   points->y = ay*sol[n]*sol[n]*sol[n] + by*sol[n]*sol[n] + cy*sol[n] + p0.y;
-            if ( points->y >= min && points->y <= max)
-            {	points->x = pl0.x;
+        else //if ( Diff(dx, 0) > TOLERANCE )
+        {   points->x = ax*sol[n]*sol[n]*sol[n] + bx*sol[n]*sol[n] + cx*sol[n] + p0.x;
+            if ( points->x >= min && points->x <= max)
+            {	points->y = m*points->x+x0;
                 points++;
                 numSol++;
             }
@@ -1552,10 +1567,10 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
             double	extremas[2];
 
             /* test if point lay in an extrema */
-            if ( Diff(dx, 0) > TOLERANCE )
+            if ( Diff(dx, 0) < TOLERANCE || (Diff(dx, 0) < Diff(dy, 0) && Diff(dx, 0) < 5.0*TOLERANCE) )
+                extremaCnt = svExtrema3(ax, bx, cx, extremas); // vertical line
+            else // if ( Diff(dx, 0) > TOLERANCE )
                 extremaCnt = svExtrema3(m*ax-ay, m*bx-by, m*cx-cy, extremas);
-            else
-                extremaCnt = svExtrema3(ax, bx, cx, extremas);
 
             for (i=0; i<extremaCnt; i++)
                 if ( Diff(sol[n], extremas[i]) < 0.00001 )
@@ -1631,7 +1646,9 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
     return distance;
 }
 
-// Fix me -> es werden nur die start/end punkte der curves zur anderen gescheckt
+/* FIXME: es werden nur die start/end punkte der curves zur anderen gescheckt
+ * modified: 2008-02-25 (compare (&iPoint, pc, &pc1[0/3]) not pc[0/3])
+ */
 - (float)sqrDistanceCurve:(NSPoint*)pc1
 {   float	distance, dist;
     NSPoint	pc[4], iPoint;
@@ -1640,9 +1657,9 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
     distance = pointOnCurveNextToPoint(&iPoint, pc1, &p0); // p0 to line
     if ((dist = pointOnCurveNextToPoint(&iPoint, pc1, &p3)) < distance) // p3 to line
         distance = dist;
-    if ((dist = pointOnCurveNextToPoint(&iPoint, pc, &pc[0])) < distance) // start line to curve
+    if ((dist = pointOnCurveNextToPoint(&iPoint, pc, &pc1[0])) < distance) // start line to curve
         distance = dist;
-    if ((dist = pointOnCurveNextToPoint(&iPoint, pc, &pc[3])) < distance) // start line to curve
+    if ((dist = pointOnCurveNextToPoint(&iPoint, pc, &pc1[3])) < distance) // start line to curve
         distance = dist;
     return distance;
 }
@@ -1985,7 +2002,8 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
     {
 	if ( VHFIsDrawingToScreen() && isSelected )
         {
-            [super drawKnobs:rect direct:direct scaleFactor:scaleFactor];
+            //[super drawKnobs:rect direct:direct scaleFactor:scaleFactor];
+            [super drawControls:rect direct:direct scaleFactor:scaleFactor];    // 2008-02-07
             [NSBezierPath setDefaultLineWidth:1.0/scaleFactor];
             [NSBezierPath strokeLineFromPoint:p0 toPoint:p1];
             [NSBezierPath strokeLineFromPoint:p3 toPoint:p2];
@@ -2450,11 +2468,11 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
     if (flatness > 10000.0)
         flatness = 10000.0;
 
-    (apPointList[0]) = malloc((ldexp(3, 9)+1) * sizeof(NSPoint));
-    (apPointList[1]) = malloc((ldexp(3, 9)+1) * sizeof(NSPoint));
+    apPointList[0] = malloc((ldexp(3, 9)+1) * sizeof(NSPoint));
+    apPointList[1] = malloc((ldexp(3, 9)+1) * sizeof(NSPoint));
     apPointList[2] = 0;
 
-    *apPointList[0]     = p0;
+    * apPointList[0]    = p0;
     *(apPointList[0]+1) = p1;
     *(apPointList[0]+2) = p2;
     *(apPointList[0]+3) = p3;
@@ -2733,7 +2751,7 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
 {   NSMutableArray	*splitList = nil;
     int			i, cnt;
     float		v0;
-    NSPoint		last, *ps;
+    NSPoint		last, *ps, crv[4];
     double		*tValues, t, t0, t1, matrix[3][3], x123[3], y123[3], abcX[3], abcY[3];
     int			n1, n2;
     NSRect		bounds;
@@ -2743,11 +2761,13 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
     ps = malloc((iCnt+2) * sizeof(NSPoint));
     //bounds = [self bounds];
     bounds = [self coordBounds];
-    bounds.origin.x -= TOLERANCE; bounds.origin.y -= TOLERANCE;
-    bounds.size.width += 2.0*TOLERANCE; bounds.size.height += 2.0*TOLERANCE;
+    bounds.origin.x -= 20*TOLERANCE; bounds.origin.y -= 20*TOLERANCE;   // same like in intersectCurves()
+    bounds.size.width += 40.0*TOLERANCE; bounds.size.height += 40.0*TOLERANCE;
+    crv[0] = p0; crv[1] = p1; crv[2] = p2; crv[3] = p3;
 
     for (i=0, cnt=0; i<iCnt; i++)
-        if ( NSPointInRect(pArray[i] , bounds) && !pointInArray(pArray[i], ps, cnt) )
+        if ( NSPointInRect(pArray[i] , bounds) && !pointInArray(pArray[i], ps, cnt) &&
+             pointOnCurveNextToPoint(&last, crv, &pArray[i]) <= 50.0*TOLERANCE )
             ps[cnt++] = pArray[i];
 
     if (!cnt)
