@@ -1,11 +1,11 @@
 /* VLine.m
  * 2-D Line object
  *
- * Copyright (C) 1996-2005 by vhf interservice GmbH
+ * Copyright (C) 1996-2006 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-29
- * modified: 2005-04-25 (clean up)
+ * modified: 2006-01-17
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -550,13 +550,13 @@
 }
 
 /* created:  1995-10-21
- * modified: 2001-11-25
+ * modified: 2006-01-17
  * purpose:  draw the graphic rotated around cp
  */
 - (void)drawAtAngle:(float)angle withCenter:(NSPoint)cp in:view
 {
     [color set];
-    [NSBezierPath setDefaultLineWidth:0.0];
+    [NSBezierPath setDefaultLineWidth:1.0/[view scaleFactor]];
     [NSBezierPath strokeLineFromPoint:vhfPointRotatedAroundCenter(p0, -angle, cp)
                               toPoint:vhfPointRotatedAroundCenter(p1, -angle, cp)];
 }
@@ -591,9 +591,11 @@
 
 /*
  * draws the line
+ * modified: 2007-11-20
  */
 - (void)drawWithPrincipal:principal
 {   NSColor	*oldColor = nil;
+    float	defaultWidth;
 
     /* colorSeparation */
     if (!VHFIsDrawingToScreen() && [principal separationColor])
@@ -605,10 +607,12 @@
 
     [super drawWithPrincipal:principal];
 
-    [NSBezierPath setDefaultLineWidth:width];
+    defaultWidth = [NSBezierPath defaultLineWidth];
+    [NSBezierPath setDefaultLineWidth:(width > 0.0) ? width : defaultWidth];
     [NSBezierPath setDefaultLineCapStyle:NSRoundLineCapStyle];
     [NSBezierPath setDefaultLineJoinStyle:NSRoundLineJoinStyle];
     [NSBezierPath strokeLineFromPoint:p0 toPoint:p1];
+    [NSBezierPath setDefaultLineWidth:defaultWidth];
 
     if ([principal showDirection])
         [self drawDirectionAtScale:[principal scaleFactor]];
@@ -826,17 +830,17 @@
 
     knobRect.origin.x = p0.x - controlsize/2.0;
     knobRect.origin.y = p0.y - controlsize/2.0;
-    if (!NSIsEmptyRect(NSIntersectionRect(hitRect , knobRect)))
+    if ((selectedKnob < 0 || selectedKnob == 1) && !NSIsEmptyRect(NSIntersectionRect(hitRect , knobRect)))
     {	*pt = p0;
-        selectedKnob = 0;
+        //selectedKnob = 0;
         return YES;
     }
 
     knobRect.origin.x = p1.x - controlsize/2.0;
     knobRect.origin.y = p1.y - controlsize/2.0;
-    if (!NSIsEmptyRect(NSIntersectionRect(hitRect , knobRect)))
+    if (selectedKnob <= 0 && !NSIsEmptyRect(NSIntersectionRect(hitRect , knobRect)))
     {	*pt = p1;
-        selectedKnob = 1;
+        //selectedKnob = 1;
         return YES;
     }
 
