@@ -1,7 +1,7 @@
 /* VMark.m
  * Drill marker or any other marking
  *
- * Copyright (C) 1996-2005 by vhf interservice GmbH
+ * Copyright (C) 1996-2007 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1997-11-13
@@ -73,20 +73,20 @@
 {
 }
 
-- (NSString*)title		{ return @"Mark"; }
+- (NSString*)title                  { return @"Mark"; }
 
-- (void)setDiameter:(float)dia	{ diameter = dia; }
-- (float)diameter		{ return diameter; }
-- (void)setOrigin:(NSPoint)pt	{ origin = pt; }
-- (NSPoint)origin		{ return origin; }
+- (void)setDiameter:(float)dia      { diameter = dia; }
+- (float)diameter                   { return diameter; }
+- (void)setOrigin:(NSPoint)pt       { origin = pt; }
+- (NSPoint)origin                   { return origin; }
 
-- (void)set3D:(BOOL)flag	{ is3D = flag; }
-- (BOOL)is3D			{ return is3D; }
-- (void)setZ:(float)v		{ z = v; is3D = YES; }
-- (float)z			{ return z; }
+- (void)set3D:(BOOL)flag            { is3D = flag; }
+- (BOOL)is3D                        { return is3D; }
+- (void)setZ:(float)v               { z = v; is3D = YES; }
+- (float)z                          { return z; }
 
-- (void)setName:(NSString*)newName { [name release]; name = [newName retain]; }
-- (NSString*)name		{ return name; }
+- (void)setName:(NSString*)newName  { [name release]; name = [newName retain]; }
+- (NSString*)name                   { return name; }
 
 #define CREATEEVENTMASK NSLeftMouseDraggedMask|NSLeftMouseDownMask|NSLeftMouseUpMask|NSPeriodicMask
 - (BOOL)create:(NSEvent *)event in:(DocView*)view
@@ -128,8 +128,8 @@
     return YES;
 }
 
-/* created:   13.11.97
- * modified:  2001-11-25
+/* created:   1997-11-13
+ * modified:  2006-01-17
  * parameter: x, y	the angles to rotate in x/y direction
  *            p		the point we have to rotate around
  * purpose:   draw the graphic rotated around p with x and y
@@ -139,7 +139,7 @@
     float	radius = 5.0;
 
     [color set];
-    [NSBezierPath setDefaultLineWidth:0.0];
+    [NSBezierPath setDefaultLineWidth:1.0/[view scaleFactor]];
     p = vhfPointRotatedAroundCenter(origin, -angle, cp);
     [NSBezierPath strokeLineFromPoint:NSMakePoint(p.x-radius, p.y)
                               toPoint:NSMakePoint(p.x+radius, p.y)];
@@ -178,9 +178,12 @@
 - (void)drawWithPrincipal:principal
 {   float	radius = 5.0;	// FIXME: shouldn't there exist a mode to display the diameter ???
     //float	radius = (displayDia) ? MMToInternal(diameter) : 5.0;
+    //float   defaultWidth = [NSBezierPath defaultLineWidth];
 
     [super drawWithPrincipal:principal];	// color
-    [NSBezierPath setDefaultLineWidth:0.0];
+    if (!VHFIsDrawingToScreen())
+        [NSBezierPath setDefaultLineWidth:0.5];
+    [NSBezierPath setDefaultLineWidth:[NSBezierPath defaultLineWidth]];
     [NSBezierPath strokeLineFromPoint:NSMakePoint(origin.x-radius, origin.y)
                               toPoint:NSMakePoint(origin.x+radius, origin.y)];
     [NSBezierPath strokeLineFromPoint:NSMakePoint(origin.x, origin.y-radius)
