@@ -1,11 +1,13 @@
 /* VText.m
  * 2-D Text object
  *
- * Copyright (C) 1996-2005 by vhf interservice GmbH
+ * Copyright (C) 1996-2009 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-02-15
- * modified: 2005-09-01 (replaceSubstring:withString:)
+ * modified: 2009-02-12 (-textDidEndEditing: add correction to height to make text appear safely on Apple)
+ *           2009-02-11 (-getFlattenedObjectAt: set yOffset for mirrored text)
+ *           2009-01-27 (-textDidEndEditing: remove empty text)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -432,10 +434,27 @@ static void initClassVars()
     viewBounds = [view bounds];
     dirty = YES;
 
+    /* Here we keep the frame of the editview always 100%, whatever happens */
+    /*{   NSRect  editFrame = [editView frame], newFrame = [view frame];
+
+        editView = [view editView];
+        newFrame.size.width = NSWidth([view frame]) / [view scaleFactor];
+        printf("editView = %.1f bounds = %.1f  newFrame = %.1f  scale = %.1f\n", editFrame.size.width, viewBounds.size.width, newFrame.size.width, [view scaleFactor]);
+        if ( Diff(newFrame.size.width, editFrame.size.width) > 0.1 )
+        {
+            newFrame.size.height = NSHeight([view frame]) / [view scaleFactor];
+            NSLog(@"Note for createText: editView corrected to view size {%.1f %.1f} -> {%.1f %.1f}",
+                  editFrame.size.width, editFrame.size.height, newFrame.size.width, newFrame.size.height);
+            [[view editView] setFrame:newFrame];
+        }
+    }*/
+
     return NSMouseInRect(bounds.origin, viewBounds, NO);
 }
 
 /*
+ * modified: 2006-09-20 (scroll text to visible)
+ *
  * Here we are going to use the shared field editor for the window to
  * edit the text in the VText.  First, we must end any other editing
  * that is going on with the field editor in this window using endEditingFor:.
@@ -609,6 +628,7 @@ static void initClassVars()
 	[fe setSelectedRange:(NSRange){0,0}];
 
     [fe toggleRuler:self];
+    [view scrollRectToVisible:frame];	// if ruler covers text -> scroll text to visible
 
     return YES;
 }
@@ -1211,7 +1231,7 @@ static void initClassVars()
 }
 
 - (NSRect)textBox		{ return bounds; }
-- (void)setTextBox:(NSRect)rect	{ bounds = rect; dirty = YES; }
+- (void)setTextBox:(NSRect)rect	{ bounds = rect; [self kernToFitHorizontal]; dirty = YES; } // 2008-02-05
 
 /*
  * Depending on the pt_num passed in, return the rectangle
@@ -1475,15 +1495,16 @@ static void initClassVars()
  * We must remove the Text object from the view heirarchy and, since
  * this Text object is going to be reused, we must set its delegate
  * back to nil.
- *
  * For further explanation of the two "ack!" lines, see edit:in: above.
+ *
+ * modified: 2009-02-12
  */
 - (void)textDidEndEditing:(NSNotification *)notification
-{   NSTextView		*fe = [notification object];
-    NSRect		oldBounds;
-    NSPoint		o;
-    float		correction = 2.0;	/* ? we use this to increase the bounds to contain the text */
-    NSAttributedString	*as;
+{   NSTextView          *fe = [notification object];
+    NSRect              oldBounds;
+    NSPoint             o;
+    float               correction = 2.0;	// ? we use this to increase the bounds that contain the text
+    NSAttributedString  *as;
 
     if ( fe )
     {
@@ -1493,12 +1514,12 @@ static void initClassVars()
         if ([fe isRulerVisible])
             [fe toggleRuler:self];
 
-        if ( ![[fe string] length] )
-            [fe setString:@"*"];
-
-        if ( [[fe string] length] )
+        if ( [[fe string] length] || attributedString )
         {   id	change = nil;
 
+            if ( ![[fe string] length] )
+                [fe setString:@"EMPTY TEXT"];
+
             if ( attributedString )
             {   change = [[EndEditingGraphicsChange alloc] initGraphicView:graphicView graphic:self];
                 correction = 0.0;
@@ -1515,7 +1536,8 @@ static void initClassVars()
             [fe sizeToFit];
             bounds = [fe frame];
             bounds.size = [fe bounds].size;
-            bounds.size.width += correction;
+            bounds.size.width  += correction;
+            bounds.size.height += correction;   // origin = upper/left here!
             bounds = [editView convertRect:bounds toView:graphicView];  // ack!
             //bounds = [graphicView centerScanRect:bounds];	/* center pixels */
             //redrawRect = NSUnionRect(bounds, redrawRect);
@@ -1529,8 +1551,9 @@ static void initClassVars()
             //[[graphicView window] disableFlushWindow];
             o = bounds.origin;
             o.y += bounds.size.height;
-            vhfRotatePointAroundCenter(&bounds.origin, o, rotAngle); /* new origin */
+            vhfRotatePointAroundCenter(&bounds.origin, o, rotAngle); // new origin
             oldBounds = NSUnionRect(bounds, oldBounds);
+            [[graphicView layerOfGraphic:self] updateObject:self];  // 2008-02-05
             [(DocView*)graphicView cache:oldBounds];
             //[graphicView draw:oldBounds];	/* update cache */
             //[(NSView*)graphicView lockFocus];
@@ -1596,7 +1619,16 @@ static void initClassVars()
 
 #ifdef __APPLE__
     {   NSBezierPath	*bezierPath = [NSBezierPath bezierPath];
-        NSFont		*fnt;
+        NSFont          *fnt;
+        float           yOffset = 0.0;
+
+        /* workaround for offset in older OS X versions, not knowing exactly when it was fixed
+         * It seems vertical centered text needs this offset.
+         */
+        if ( centerVertical /*|| NSAppKitVersionNumber < 824.42*/ )   // Mac OS X < 10.4.10
+            yOffset = Max(0.0, (bounds.size.height-rect.size.height-1.0)/2.0);
+        if ( aspectRatio < 0.0 )    // mirrored text
+            yOffset -= bounds.size.height; // Apple needs that
 
         cnt = [lm numberOfGlyphs];
         if (cnt > 1)
@@ -1609,14 +1641,16 @@ static void initClassVars()
             p = [lm locationForGlyphAtIndex:i];
             [bezierPath removeAllPoints];
             [bezierPath moveToPoint:NSMakePoint(bRect.origin.x+rect.origin.x+p.x,
-                                                bRect.origin.y+bRect.size.height-rect.origin.y-p.y)];
+                                                bRect.origin.y+bRect.size.height-rect.origin.y-p.y-yOffset)];
             fnt = [[drawText textStorage] attribute:NSFontAttributeName atIndex:i effectiveRange:NULL];
             [bezierPath appendBezierPathWithGlyph:[lm glyphAtIndex:i] inFont:fnt];
             pathG = [VPath pathWithBezierPath:bezierPath];
             [pathG setWidth:0.0];
+            //printf("y=%.0f h=%.0f lh=%.0f  oberkannte=%.0f  vorstellung=%.0f\n", bRect.origin.y, bRect.size.height, [self lineHeight], bRect.origin.y+bRect.size.height, bRect.origin.y+bRect.size.height-([self lineHeight]*aspectRatio/2.0));
+             /* /2.0 war bis 2007-10-26, tut nicht mit 10.4.10 (824.42) */
             if (aspectRatio != 1.0)
                 [pathG scale:1.0 :aspectRatio
-                  withCenter:NSMakePoint(1.0, bRect.origin.y+bRect.size.height/2.0)];
+                  withCenter:NSMakePoint(1.0, bRect.origin.y+bRect.size.height/1.0)];
             [pathG setAngle:-rotAngle withCenter:bRect.origin];
             /* FIXME: stroked fonts shouldn't be filled! */
             [pathG setFilled:YES optimize:NO];
@@ -1796,15 +1830,16 @@ static void initClassVars()
     [super initFromPropertyList:plist inDirectory:directory];
     rotAngle = [plist floatForKey:@"rotAngle"];
     aspectRatio = [plist floatForKey:@"aspectRatio"];
-    [self setRichTextData:[plist objectForKey:@"data"]];
+    [self setRichTextData:[plist objectForKey:@"data"]];    // this also loads the text colors for paragraphs !
     bounds = rectFromPropertyList([plist objectForKey:@"bounds"]);
     isSerialNumber = ([plist objectForKey:@"isSerialNumber"] ? YES : NO);
     fitHorizontal = ([plist objectForKey:@"fitHorizontal"] ? YES : NO);
     centerVertical = ([plist objectForKey:@"centerVertical"] ? YES : NO);
     [self font];
 
-    if (!(fillColor = colorFromPropertyList([plist objectForKey:@"fillColor"], [self zone])))
-        [self setFillColor:[color copy]];
+    fillColor = colorFromPropertyList([plist objectForKey:@"fillColor"], [self zone]);
+    /*if (!(fillColor = colorFromPropertyList([plist objectForKey:@"fillColor"], [self zone])))
+        [self setFillColor:[color copy]];   // destroys paragraph colors !*/
     if (!(endColor = colorFromPropertyList([plist objectForKey:@"endColor"], [self zone])))
         [self setEndColor:[NSColor blackColor]];
     graduateAngle = [plist floatForKey:@"graduateAngle"];
