@@ -1,11 +1,11 @@
 /* DXFImport.m
  * DXF import object
  *
- * Copyright (C) 1996-2004 by vhf interservice GmbH
+ * Copyright (C) 1996-2009 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-05-01
- * modified: 2004-08-04
+ * modified: 2007-01-30 (close PolyLine: check for start == end point)
  *
  * This file is part of the vhf Import Library.
  *
@@ -243,7 +243,7 @@ static NSArray *getTableFromData(NSString *data)
 
         /* scan id */
         if ( ![scanner scanInt:&ident] )
-        {   NSLog(@"id expected at location: %d", [scanner scanLocation]);
+        {   NSLog(@"DXF-Import: id expected at location: %d", [scanner scanLocation]);
             return NO;
         }
         gotoNextLine(scanner);	// goto value line
@@ -280,11 +280,11 @@ static NSArray *getTableFromData(NSString *data)
     return table;
 }
 
-/* created:   2001-02-23
- * modified: 
- *
- * purpose:   get visible objects from OBJECTS section
+/* get visible objects from OBJECTS section
  * parameter: data (dxf file)
+ *
+ * created:  2001-02-23
+ * modified: 2005-11-23 (check for EOF)
  */
 static NSArray *getVisibleListFromData(NSString *data)
 {   NSScanner		*scanner = [NSScanner scannerWithString:data];
@@ -306,18 +306,20 @@ static NSArray *getVisibleListFromData(NSString *data)
 
         /* scan id */
         if ( ![scanner scanInt:&ident] )
-        {   NSLog(@"id expected at location: %d", [scanner scanLocation]);
+        {   NSLog(@"DXF-Import: id expected at location: %d", [scanner scanLocation]);
             return nil;
         }
         gotoNextLine(scanner);	// goto value line
         switch ( ident )
         {
-            case ID_GROUP:
+            case ID_GROUP:	// 0
                 if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
                     string = @"";
+                if ([string isEqual:GRPEOF])
+                    return nil;
                 idbuffer = ([string isEqual:GRPIDBUFFER]) ? YES : NO;
                 break;
-            case ID_REFENTITY:
+            case ID_REFENTITY:	// 330
                 if (!idbuffer)
                     break;
                 if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
@@ -549,10 +551,11 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 }
 
 /* we need scanner on the id line of a group
+ * modified: 2005-09-26 (close polyline with arc)
  */
 - (BOOL)getGraphicFromData:(NSScanner*)scanner :(id)cList
-{   NSPoint	p0, p1;
-    NSString	*name;
+{   NSPoint     p0, p1;
+    NSString    *name;
 
     /* goto name line */
     [scanner scanUpToCharactersFromSet:newLineSet intoString:NULL];
@@ -565,7 +568,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 
     switch ( state.mode )
     {
-	case MODE_NORMAL:
+        case MODE_NORMAL:
             /* line */
             if ( [name isEqual:GRPLINE] )
             {
@@ -647,24 +650,29 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
                 float	a;
 
                 [self getVertex:scanner :&kind :&p0 :&p1 :&a];
+                //printf("kind=%d p0={%.0f %.0f} p1={%.0f %.0f} a=%.0f\n", kind, p0.x, p0.y, p1.x, p1.y, a);
                 switch (kind)
                 {
-                    case 1: [self addLine:p0 :p1 toList:cList]; break;
+                    case 1: [self addLine:p0 :p1   toList:cList]; break;
                     case 2: [self addArc:p1 :p0 :a toList:cList]; break;
                 }
             }
             else if ( [name isEqual:GRPSEQEND] )
             {
                 if (state.modeClosed)
-                {   //NXPoint	ctr;
-                    //NXCoord	angle;
+                {   NSPoint	ctr;
+                    float	angle;
 
-                    //if (!state.A)
-                    [self addLine:state.first :state.point toList:cList];
-                    //else
-                    //{	buildArc(state.point, state.first, state.A, &ctr, &angle);
-                    //	[self addArc:ctr :state.first :angle toList:cList];
-                    //}
+                    if ( Diff(state.first.x, state.point.x) > TOLERANCE ||
+                         Diff(state.first.y, state.point.y) > TOLERANCE )
+                    {
+                        if (!group.a /* state.A*/)
+                            [self addLine:state.first :state.point toList:cList];
+                        else
+                        {   buildArc(state.point, state.first, group.a, &ctr, &angle);
+                            [self addArc:ctr :state.point :angle toList:cList];
+                        }
+                    }
                 }
                 if ([cList count])
                 {   [self addStrokeList:cList toLayer:group.layer];
@@ -731,7 +739,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 }
 
 /* created:   2001-02-22
- * modified:  
+ * modified:  2006-08-10 (check for close with & operator)
  *
  * purpose:   get LWPOLYLINE group
  * parameter: scanner (the dxf data at id line)
@@ -760,7 +768,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 
         /* scan id */
         if ( ![scanner scanInt:&i] )
-        {   NSLog(@"id expected at location: %d", [scanner scanLocation]);
+        {   NSLog(@"DXF-Import: id expected at location: %d", [scanner scanLocation]);
             return NO;
         }
         gotoNextLine(scanner);	// goto value line
@@ -885,7 +893,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
             case ID_FLAGS:
                 if (![scanner scanInt:&group.flags])
                     group.flags = 0;
-                state.modeClosed = ( group.flags ==  1 ) ? MODE_CLOSED : 0;
+                state.modeClosed = ( group.flags &  1 ) ? MODE_CLOSED : 0;
                 break;
         }
         gotoNextLine(scanner);	// goto id line
@@ -937,7 +945,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
     float	bw, ew;
     NSPoint	beg, end;
 
-    a = group.a;
+    a = group.a;	// type is one behind
     [self scanGroup:scanner];
 
     state.color = [self currentColor];
@@ -1130,6 +1138,7 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
     state.color = [self currentColor];
     ctr->x = DeviceResToInternal(group.x0, res) + state.offset.x;
     ctr->y = DeviceResToInternal(group.y0, res) + state.offset.y;
+    //FIXME: group.extX, group.extY, group.extZ
     r = DeviceResToInternal(group.width, res);
     ba = group.begAngle;
     ea = group.endAngle;
@@ -1361,7 +1370,8 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
 /* get group entries
  * scanner starts at id line of first entry
  * scanner scans until it reaches the next group
- * created:  19.03.97
+ * created:  1997-03-19
+ * modified: 2009-02-06 (read in extrusion direction)
  */
 - (BOOL)scanGroup:(NSScanner*)scanner
 {
@@ -1379,13 +1389,14 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
     group.x3 = group.y3 = 0.0;
     group.a = 0.0;
     group.flags = group.genFlags = 0;
+    group.extX = group.extY = 0.0; group.extZ = -1.0;
 
     while ( ![scanner isAtEnd] )
     {	int	i, location = [scanner scanLocation];
 
         /* scan id */
         if ( ![scanner scanInt:&i] )
-        {   NSLog(@"id expected at location: %d", [scanner scanLocation]);
+        {   NSLog(@"DXF-Import: id expected at location: %d", [scanner scanLocation]);
             return NO;
         }
         gotoNextLine(scanner);	// goto value line
@@ -1500,7 +1511,21 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
                 if (![scanner scanInt:&group.numGrp])
                     group.numGrp = 0;
                 break;
-           case ID_GROUP:
+            case ID_EXT_X:
+                if (![scanner scanFloat:&group.extX])
+                    group.extX = 0.0;
+                break;
+            case ID_EXT_Y:
+                if (![scanner scanFloat:&group.extY])
+                    group.extY = 0.0;
+                if ( group.extX != 0.0 || group.extY != 0.0 )   // FIXME: extrusion direction
+                    NSLog(@"DXF-Import: extrusion direction not supported. File-Location = %d", [scanner scanLocation]);
+                break;
+            case ID_EXT_Z:
+                if (![scanner scanFloat:&group.extZ])
+                    group.extZ = 1.0;
+                break;
+            case ID_GROUP:
                 [scanner setScanLocation:location];
                 return YES;
             default:
