@@ -1,11 +1,11 @@
 /* VHFStringAdditions.m
  * vhf NSString additions
  *
- * Copyright (C) 1997-2003 by vhf interservice GmbH
+ * Copyright (C) 1997-2008 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1997-07-08
- * modified: 2003-06-26
+ * modified: 2008-11-08 (inactive method for NSMutableArray parked here)
  *
  * This file is part of the vhf Shared Library.
  *
@@ -37,6 +37,89 @@
 
 @implementation NSString(VHFStringAdditions)
 
+/* return string from float with 4 decimals
+ */
++ (NSString*)stringWithFloat:(float)value
+{   int			i;
+    NSMutableString	*string;
+    NSRange		range;
+    char		c;
+
+    value += 0.000005;
+    string = [NSMutableString stringWithFormat:@"%.4f", value];
+
+    /* avoid 0 as last character */
+    for (i=[string length]-1; i && [string characterAtIndex:i] == '0'; i--)
+    {	range.location = i;
+        range.length = 1;
+        [string deleteCharactersInRange:range];
+    }
+
+    /* avoid point as last character */
+    c = [string characterAtIndex:[string length]-1];
+    if ( c == '.' || c == ',' )
+    {	range.location = [string length]-1;
+        range.length=1;
+        [string deleteCharactersInRange:range];
+    }
+
+    /* workaround for OpenStep */
+    if ( [NSDecimalSeparator isEqual:@","] )
+        return [string stringByReplacing:@"." by:@","];
+
+    return string;
+}
+
+/* created:      2006-02-24 (2004-12-12)
+ * modified:     2004-12-12
+ * purpose:      build a string from a value
+ *               'string' is a representation of 'value' with a maximum of n decimals
+ *               the value will stay within limits
+ *               '.' and ',' are regarded as decimal points, there are no thousands separators !
+ * parameter:    value
+ *               limits
+ *               decimals
+ * return value: string
+ */
++ (NSString*)stringWithFloat:(float)value decimals:(int)decimals
+{   int			i;
+    NSString		*format = [NSString stringWithFormat:@"%%.%df", decimals];
+    NSMutableString	*string;
+    NSRange		range;
+    char		c;
+
+    value += 0.000005;
+
+    //if (value < limits.min)
+    //    value = limits.min;
+    //if (value > limits.max)
+    //    value = limits.max;
+    string = [NSMutableString stringWithFormat:format, value];
+
+    /* avoid 0 as last character */
+    for (i=[string length]-1; i && [string characterAtIndex:i]=='0'; i--)
+    {	range.location = i;
+        range.length=1;
+        [string deleteCharactersInRange:range];
+    }
+
+    /* avoid point as last character */
+    c = [string characterAtIndex:[string length]-1];
+    if ( c == '.' || c == ',' )
+    {	range.location = [string length]-1;
+        range.length=1;
+        [string deleteCharactersInRange:range];
+    }
+
+    /* workaround for OpenStep */
+    if ( [NSDecimalSeparator isEqual:@","] )
+        return [string stringByReplacing:@"." by:@","];
+
+    return string;
+}
+
+
+
 - (NSString*)stringByRemovingTrailingCharacters:(NSString*)chars
 {   NSRange	range;
     int		ix = [self length];
@@ -165,8 +248,9 @@
     return string;
 }
 
-/* created: 16.02.99
- * < 0 -> left aligned
+/* created: 1999-02-16
+ * < 0 -> right aligned
+ * > 0 -> left  aligned
  */
 - (NSString*)stringWithLength:(int)length
 {
@@ -191,7 +275,7 @@
     if ( (int)[self length] > length )
         return [self substringToIndex:length];
     /* too short */
-    newString = [NSMutableString string];
+    newString = (NSMutableString*)[NSMutableString string];
     space = length - [self length];
     for ( i=0; i<space; i++ )
         [newString appendString:fillChar];
@@ -214,6 +298,17 @@
     return num;
 }
 
+/* created: 2004-05-22
+ */
+- (int)countOfCharacter:(unsigned char)c inRange:(NSRange)range
+{   int	i, num = 0;
+
+    for (i=range.location; i<range.location+range.length; i++)
+        if ( [self characterAtIndex:i] == c )
+            num++;
+    return num;
+}
+
 
 
 /* created:  1997-05-31
@@ -245,20 +340,20 @@
  *
  * wildcards =
  * {
- *     skipNum = "";	// "#"       skip digits (0123456789)
- *     skipAll = "";	// "~"       skip any characters
- *     skipSpace = "";	// "_"       skip space
- *     skipChar = "";	// "?"       skip single character
- *     skipChars = "";	// "|"       skip range of characters containing chars
- *     chars = "";	// ".,- \t"  characters to be skipped
+ *     skipNum   = "#";         // "#"       skip digits (0123456789)
+ *     skipAll   = "~";         // "~"       skip any characters
+ *     skipSpace = "_";         // "_"       skip space
+ *     skipChar  = "?";         // "?"       skip single character
+ *     skipChars = "|";         // "|"       skip range of characters containing chars
+ *     chars     = ".,- \t";    // ".,- \t"  characters to be skipped
  * }
  */
 - (NSRange)rangeOfSequence:(NSString*)sequence options:(int)options
                      range:(NSRange)sRange wildcards:(NSDictionary*)wildcards
 {   NSMutableArray	*tokens = [NSMutableArray array];
-    NSString		*skipNum = [wildcards objectForKey:@"skipNum"];
-    NSString		*skipAll = [wildcards objectForKey:@"skipAll"];
-    NSString		*skipChar = [wildcards objectForKey:@"skipChar"];
+    NSString		*skipNum   = [wildcards objectForKey:@"skipNum"];
+    NSString		*skipAll   = [wildcards objectForKey:@"skipAll"];
+    NSString		*skipChar  = [wildcards objectForKey:@"skipChar"];
     NSString		*skipSpace = [wildcards objectForKey:@"skipSpace"];
     NSString		*skipChars = [wildcards objectForKey:@"skipChars"];
     NSMutableString	*mutString = [NSMutableString string];
@@ -416,4 +511,37 @@
     return seqRange;
 }
 
+#if 0   // should go into NSMutableString
+/* created: 2008-11-08
+ *
+ * wildcards:
+ * '#' -> skip number "0123456789-+."
+ * '*' -> skip all
+ * '_' -> skip white space " \t"
+ * '?' -> skip single character
+ *
+ * sequence = "In the year #"
+ */
+- (void)replaceCharactersAfterSequence:(NSString*)seqFr toSequence:(NSString*)seqTo by:(NSString*)string
+{   NSRange     range;
+    int         frIx, toIx;
+    NSDictionary	*wildcards = [NSDictionary dictionaryWithObjectsAndKeys:
+                    @"#", @"skipNum",
+                    @"*", @"skipAll",
+                    @"_", @"skipSpace",
+                    @"?", @"skipChar",
+                    @"|", @"skipChars", @"", @"chars", nil];
+
+    range = [self rangeOfSequence:seqFr options:0 /*range:NSMakeRange(0. [string length]) wildcards:wildcards*/];
+    if (!range.length)
+        return;
+    frIx = range.location+range.length;
+    range = [self rangeOfSequence:seqTo options:0 range:NSMakeRange(frIx, [self length]-frIx) /*wildcards:wildcards*/];
+    if (!range.length)
+        return;
+    toIx = range.location;
+    return [self replaceCharactersInRange:NSMakeRange(frIx, toIx-frIx) withString:string];
+}
+#endif
+
 @end
